SystemOrganization addCategory: #'Alt-Browser'!
SystemOrganization addCategory: #'Alt-Browser-Commands'!

ComposableModel subclass: #AltBrowser
	instanceVariableNames: 'treeModel methodModel textModel'
	classVariableNames: 'rootItems'
	poolDictionaries: ''
	category: 'Alt-Browser'!

!AltBrowser commentStamp: '<historical>' prior: 0!
An AltBrowser is the main application.

Instance Variables
	methodModel:		<A method model>
	treeModel:		<An ABPackageList>

methodModel
	- handle the display of methods.

treeModel
	- handle navigation between packages/category/classes/protocols/methods
!

!AltBrowser class methodsFor: 'updating' stamp: 'ThierryGoubier 7/27/2012 11:05'!
aCategoryRemoved: anEvent
	"Ok, a class category was removed."

	| node r |
	node := self rootItems
		detect: [ :e | (r := e findClassCategory: anEvent categorySelector) notNil ] ifNone: [^self].
	r parentNode contents remove: r.
	self allInstances do: [ :l | l removeNode: r ]! !

!AltBrowser class methodsFor: 'updating' stamp: 'ThierryGoubier 7/27/2012 11:31'!
aClassRemoved: anEvent
	"Ok, a class was removed."

	| node r |
	node := self rootItems
		detect: [ :e | (r := e findClass: anEvent removedClass) notNil ] ifNone: [^self].
	r parentNode contents remove: r.
	self allInstances do: [ :l | l removeNode: r ]! !

!AltBrowser class methodsFor: 'updating' stamp: 'ThierryGoubier 7/26/2012 21:50'!
aMCWorkingCopyCreated: anEvent
	"Uncool: we need a MCWorkingCopy, and the event gives us a MCPackage :-(."

	| package cat node |
	package := MCWorkingCopy allManagers detect: [ :e | e package == anEvent package ].
	cat := package packageName copyUpTo: $-.
	node := self rootItems detect: [ :e | e item key = cat ] ifNone: [ nil ].
	node notNil
		ifTrue: [ node item value detect: [ :e | e == package ] ifNone: [ node item value add: package ] ]
		ifFalse: [ 
			node := ABPackageCategoryNode with: cat -> (OrderedCollection with: package).
			self rootItems add: node.
			self allInstances do: [ :l | l addPackageCategory: node ] ]! !

!AltBrowser class methodsFor: 'updating' stamp: 'ThierryGoubier 7/27/2012 10:45'!
aMCWorkingCopyDeleted: anEvent
	"Uncool: we need a MCWorkingCopy, and the event gives us a MCPackage :-(."

	| package cat node r |
	node := self rootItems
		detect: [ :e | (r := e item value detect: [ :p | p package == anEvent package ] ifNone: [ nil ]) notNil ].
	node item value remove: r.
	self allInstances do: [ :l | l removeNode: r ].
	node item value isEmpty
		ifTrue: [ 
			self rootItems remove: node.
			self allInstances do: [ :l | l removeNode: node ] ]! !

!AltBrowser class methodsFor: 'updating' stamp: 'ThierryGoubier 7/27/2012 15:33'!
aMethodAdded: anEvent
	"Ok, a method was added. Does not mean modify."

	| node r s c |
	node := self rootItems detect: [ :e | (c := e findClass: anEvent itemClass) notNil ] ifNone: [ ^ self ].
	c hasContents
		ifTrue: [ 
			(r := c findMethodCategory: anEvent itemProtocol) isNil
				ifTrue: [ ^ self ].
			r hasContents
				ifTrue: [ 
					r contents add: (s := r childNodeFromItem: anEvent itemSelector).
					self allInstances do: [ :each | each addNode: s ] ] ]! !

!AltBrowser class methodsFor: 'updating' stamp: 'ThierryGoubier 7/27/2012 16:30'!
aMethodRemoved: anEvent
	"Ok, a method was removed."

	| node r s c |
	node := self rootItems detect: [ :e | (c := e findClass: anEvent itemClass) notNil ] ifNone: [ ^ self ].
	c hasContents
		ifTrue: [ 
			(r := c findMethodCategory: anEvent itemProtocol) isNil
				ifTrue: [ ^ self ].
			r hasContents
				ifTrue: [ 
					r contents remove: (s := r findMethod: anEvent itemSelector).
					self allInstances do: [ :each | each removeNode: s ] ] ]! !

!AltBrowser class methodsFor: 'spec' stamp: 'ThierryGoubier 7/26/2012 15:22'!
defaultPackageCategoriesNames
	"As it says: categories for packages who don't use the - convention."
	
	^ #('Configuration' 'Nautilus' 'Text' 'Shout' 'Collections' 'Fuel' 'Morphic' 'Monticello' 'Tools' 'Compiler' 'Network' 'Compression' 'Balloon' 'Kernel' 'FreeType' 'SmaCC')! !

!AltBrowser class methodsFor: 'spec' stamp: 'ThierryGoubier 7/16/2012 22:39'!
defaultSpec
	<spec>
	
	^ {#ComposableSpec.
			#add:. {
				{#model. #treeModel}.
				#layout:.
				{#FrameLayout. #fractions:offsets:. (0 @ 0 corner: 0.3 @ 1). (0 @ 0 corner: 0 @ 0)}
			}.
			#add:. {
				{#model. #textModel}.
				#layout:.
				{#FrameLayout. #fractions:offsets:. (0.3 @ 0 corner: 1 @ 1). (0 @ 0 corner: 0 @ 0)}
			}
		}! !

!AltBrowser class methodsFor: 'spec' stamp: 'ThierryGoubier 7/26/2012 10:42'!
example
	"AltBrowser example"
	
	AltBrowser new openWithSpec! !

!AltBrowser class methodsFor: 'private' stamp: 'ThierryGoubier 7/26/2012 15:24'!
getRootItems
	"This is where I should rework the organisation. Use broad classes to reduce the number of top-level items. To be improved. Does not catch classes which have no "

	| l o |
	l := MCWorkingCopy allManagers asOrderedCollection.
	o := OrderedCollection new.
	self defaultPackageCategoriesNames
		do: [ :w | 
			| i |
			i := w -> OrderedCollection new.
			(l select: [ :e | e packageName size >= w size and: [ (e packageName copyFrom: 1 to: w size) = w ] ])
				do: [ :e | 
					l remove: e.
					i value add: e ].
			o add: i ].
	[ l notEmpty ]
		whileTrue: [ 
			| p w i |
			p := l first.
			w := p packageName copyUpTo: $-.
			i := w -> (OrderedCollection with: p).
			o add: i.
			l remove: p.
			(l select: [ :e | (e packageName copyUpTo: $-) = w ])
				do: [ :e | 
					l remove: e.
					i value add: e ] ].
	^ o collect: [ :e | ABPackageCategoryNode with: e ]! !

!AltBrowser class methodsFor: 'initialization' stamp: 'ThierryGoubier 7/27/2012 16:31'!
registerOnSystemAnnouncements
	"Global registration to update the list."

	"self registerOnSystemAnnouncements."

	"SystemAnnouncer announcer unsubscribe: self"

	SystemAnnouncer announcer weak
		on: MCWorkingCopyCreated send: #aMCWorkingCopyCreated: to: self;
		on: MCWorkingCopyDeleted send: #aMCWorkingCopyDeleted: to: self;
		on: CategoryRemoved send: #aCategoryRemoved: to: self;
		on: MethodAdded send: #aMethodAdded: to: self;
		on: MethodRemoved send: #aMethodRemoved: to: self! !

!AltBrowser class methodsFor: 'private' stamp: 'ThierryGoubier 7/26/2012 16:30'!
reset
	"reset the rootItems. Should propagate to all instances as well."
	
	"AltBrowser reset"

	rootItems := nil.
	self allInstances do: [ :e | e reset ]! !

!AltBrowser class methodsFor: 'accessing' stamp: 'ThierryGoubier 7/26/2012 15:26'!
rootItems
	rootItems isNil
		ifTrue: [ rootItems := self getRootItems ].
	^ rootItems! !

!AltBrowser class methodsFor: 'spec' stamp: 'ThierryGoubier 7/15/2012 22:44'!
title
	^ 'Alt Browser'! !

!AltBrowser class methodsFor: 'private' stamp: 'ThierryGoubier 7/26/2012 16:23'!
updateAll
	"Entry point for updating the whole tree if needed."
	
	self reset! !

!AltBrowser methodsFor: 'updating' stamp: 'ThierryGoubier 7/27/2012 13:55'!
addNode: aNode
	"A node has been added."

	treeModel rootsAdd: aNode! !

!AltBrowser methodsFor: 'updating' stamp: 'ThierryGoubier 7/26/2012 21:19'!
addPackageCategory: aNode
	"A new package category has been added. Update the tree by applying it."

	treeModel rootsAdd: (ABTreeNodeModel with: aNode model: aNode)! !

!AltBrowser methodsFor: 'commands' stamp: 'ThierryGoubier 7/25/2012 23:22'!
buildMenu: aMenu on: aTarget
	treeModel selectedItem notNil
		ifTrue: [ treeModel selectedItem buildTextMenu: aMenu on: aTarget with: self ].
	^ aMenu! !

!AltBrowser methodsFor: 'private' stamp: 'ThierryGoubier 7/26/2012 16:24'!
getRootItems
	^ self class rootItems collect: [ :e | ABTreeNodeModel with: e model: e ]! !

!AltBrowser methodsFor: 'initialization' stamp: 'ThierryGoubier 7/26/2012 16:40'!
initializePresenter
	"The models are created, it means I can register on events."
	
	treeModel roots: self getRootItems.
	treeModel childrenBlock: [ :it | it contents ].
	treeModel displayBlock: [ :it | it asString ].
	treeModel
		whenSelectedItemChanged: [ :sel | 
			sel isNil
				ifTrue: [ 
					textModel text: ''.
					textModel behavior: nil.
					textModel aboutToStyle: false ]
				ifFalse: [ 
					textModel text: sel sourceCode.
					textModel behavior: sel sourceClass.
					textModel aboutToStyle: sel isStyled ] ].
	textModel menu: [ :m :t | self buildMenu: m on: t ].
	self registerOnSystemAnnouncements! !

!AltBrowser methodsFor: 'initialization' stamp: 'ThierryGoubier 7/17/2012 22:43'!
initializeWidgets
	self instantiateModels: #(#treeModel #ABTreeModel #textModel #ABTextModel).
	textModel aboutToStyle: true! !

!AltBrowser methodsFor: 'initialization' stamp: 'ThierryGoubier 7/26/2012 16:42'!
registerOnSystemAnnouncements
	"Where I do register myself on announcements and track them down."

	"Problem: the rootItems list is global."! !

!AltBrowser methodsFor: 'updating' stamp: 'ThierryGoubier 7/27/2012 17:34'!
removeNode: aNode
	"A node in the tree has been removed. Update the tree by applying it."

	self selectedItem item == aNode
		ifTrue: [ self selectedItem: nil ].
	treeModel rootsRemove: aNode! !

!AltBrowser methodsFor: 'private' stamp: 'ThierryGoubier 7/26/2012 16:25'!
reset
	treeModel isNil
		ifFalse: [ treeModel roots: self getRootItems ]! !

!AltBrowser methodsFor: 'accessing' stamp: 'ThierryGoubier 7/26/2012 10:45'!
selectedItem
	^ treeModel selectedItem! !

!AltBrowser methodsFor: 'accessing' stamp: 'ThierryGoubier 7/27/2012 16:38'!
selectedItem: anItem
	treeModel setSelectedItem: anItem! !

!AltBrowser methodsFor: 'accessing' stamp: 'ThierryGoubier 7/16/2012 00:35'!
textModel
	^textModel! !

!AltBrowser methodsFor: 'accessing' stamp: 'ThierryGoubier 7/15/2012 22:41'!
treeModel
	^treeModel! !

Model subclass: #ABAbstractNode
	instanceVariableNames: 'item contents parentNode'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser'!

!ABAbstractNode commentStamp: '<historical>' prior: 0!
An ABAbstractNode is the abstract type of all nodes used by the ABPackageList.!

!ABAbstractNode class methodsFor: 'instance creation' stamp: 'ThierryGoubier 7/19/2012 22:36'!
with: anItem
	^ self new item: anItem! !

!ABAbstractNode methodsFor: 'actions' stamp: 'ThierryGoubier 7/26/2012 14:15'!
accept: aText
	"When the text editor associated has accepted, do that..."

	"That is, nothing by default."

	! !

!ABAbstractNode methodsFor: 'printing' stamp: 'ThierryGoubier 7/19/2012 22:43'!
asString
	^ item asString! !

!ABAbstractNode methodsFor: 'actions' stamp: 'ThierryGoubier 7/14/2012 11:19'!
browseItem
	"By default do nothing."! !

!ABAbstractNode methodsFor: 'querying' stamp: 'ThierryGoubier 7/25/2012 23:16'!
buildTextMenu: aMenu on: aTarget with: aRequestor
	| t |
	t := false.
	(Pragma allNamed: #textAreaCommand from: self class to: ABAbstractNode)
		do: [ :e | 
			t
				ifTrue: [ aMenu addLine ]
				ifFalse: [ t := true ].
			(self perform: e selector) do: [ :c | (c on: aTarget for: aRequestor) addItemToMenu: aMenu ] ]! !

!ABAbstractNode methodsFor: 'private' stamp: 'ThierryGoubier 7/19/2012 22:32'!
childNodeClassFromItem: anItem
	^ self class! !

!ABAbstractNode methodsFor: 'private' stamp: 'ThierryGoubier 7/19/2012 22:31'!
childNodeFromItem: anItem
	^ ((self childNodeClassFromItem: anItem) with: anItem)
		parentNode: self;
		yourself! !

!ABAbstractNode methodsFor: 'private' stamp: 'ThierryGoubier 7/19/2012 22:34'!
childrenItems
	^ #()! !

!ABAbstractNode methodsFor: 'commands' stamp: 'ThierryGoubier 7/25/2012 23:18'!
clipboardCommands
	<textAreaCommand>
	^ {ABCutTextCommand.
	ABCopyTextCommand.
	ABPasteTextCommand}! !

!ABAbstractNode methodsFor: 'accessing' stamp: 'ThierryGoubier 7/19/2012 22:32'!
contents
	contents isNil
		ifTrue: [ contents := self childrenItems collect: [ :i | self childNodeFromItem: i ] ].
	^ contents! !

!ABAbstractNode methodsFor: 'actions' stamp: 'ThierryGoubier 7/14/2012 11:19'!
doubleClick
	self browseItem! !

!ABAbstractNode methodsFor: 'commands' stamp: 'ThierryGoubier 7/25/2012 23:18'!
executeCommands
	<textAreaCommand>
	^ {ABDoItTextCommand.
	ABPrintItTextCommand.
	ABInspectItTextCommand.
	ABExploreItTextCommand.
	ABDebugItTextCommand}! !

!ABAbstractNode methodsFor: 'actions' stamp: 'ThierryGoubier 7/14/2012 11:19'!
exploreItem
	self explore! !

!ABAbstractNode methodsFor: 'querying' stamp: 'ThierryGoubier 7/27/2012 11:34'!
findClass: aClass
	"Return the node if the item matches. By default, search among the kids. Is lazy and will avoid triggering child building."

	contents isNil
		ifFalse: [ 
			| res |
			contents
				do: [ :each | 
					(res := each findClass: aClass) notNil
						ifTrue: [ ^ res ] ] ].
	^ nil! !

!ABAbstractNode methodsFor: 'querying' stamp: 'ThierryGoubier 7/27/2012 10:24'!
findClassCategory: aCat
	"Return the node if the item matches. By default, search among the kids. Is lazy and will avoid triggering child building."

	contents isNil
		ifFalse: [ 
			| res |
			contents
				do: [ :each | 
					(res := each findClassCategory: aCat) notNil
						ifTrue: [ ^ res ] ] ].
	^ nil! !

!ABAbstractNode methodsFor: 'querying' stamp: 'ThierryGoubier 7/27/2012 14:21'!
findClassComment: aClass
	"Return the node if the item matches. By default, search among the kids. Is lazy and will avoid triggering child building."

	contents isNil
		ifFalse: [ 
			| res |
			contents
				do: [ :each | 
					(res := each findClassComment: aClass) notNil
						ifTrue: [ ^ res ] ] ].
	^ nil! !

!ABAbstractNode methodsFor: 'querying' stamp: 'ThierryGoubier 7/27/2012 13:39'!
findMethod: aClass
	"Return the node if the item matches. By default, search among the kids. Is lazy and will avoid triggering child building."

	contents isNil
		ifFalse: [ 
			| res |
			contents
				do: [ :each | 
					(res := each findMethod: aClass) notNil
						ifTrue: [ ^ res ] ] ].
	^ nil! !

!ABAbstractNode methodsFor: 'querying' stamp: 'ThierryGoubier 7/27/2012 13:41'!
findMethodCategory: aMethodCategory
	"Return the node if the item matches. By default, search among the kids. Is lazy and will avoid triggering child building."

	contents isNil
		ifFalse: [ 
			| res |
			contents
				do: [ :each | 
					(res := each findMethodCategory: aMethodCategory) notNil
						ifTrue: [ ^ res ] ] ].
	^ nil! !

!ABAbstractNode methodsFor: 'testing' stamp: 'ThierryGoubier 7/27/2012 13:50'!
hasContents
	^ contents notNil! !

!ABAbstractNode methodsFor: 'actions' stamp: 'ThierryGoubier 7/14/2012 11:20'!
inspectItem
	self inspect! !

!ABAbstractNode methodsFor: 'testing' stamp: 'ThierryGoubier 7/19/2012 23:20'!
isStyled
	"Should return true if the text is to be styled (default smalltalk code)."
	
	^ false! !

!ABAbstractNode methodsFor: 'accessing' stamp: 'ThierryGoubier 7/19/2012 22:36'!
item
	^ item! !

!ABAbstractNode methodsFor: 'accessing' stamp: 'ThierryGoubier 7/19/2012 22:36'!
item: anObject
	item := anObject! !

!ABAbstractNode methodsFor: 'actions' stamp: 'ThierryGoubier 7/14/2012 11:20'!
keyStroke: anEvent from: aTreeMorph
	| c |
	c := anEvent keyCharacter.
	c = $b
		ifTrue: [ 
			self browseItem.
			^ true ].
	c = $i
		ifTrue: [ 
			self inspectItem.
			^ true ].
	c = $I
		ifTrue: [ 
			self exploreItem.
			^ true ].
	^ false! !

!ABAbstractNode methodsFor: 'actions' stamp: 'ThierryGoubier 7/14/2012 11:20'!
menu: menu shifted: b 
	menu
		add: 'Expand all' translated
		target: self
		selector: #expandAll.
	menu
		add: 'Browse (b)' translated
		target: self
		selector: #browseItem.
	menu
		add: 'Inspect (i)' translated
		target: self
		selector: #inspectItem.
	menu
		add: 'Explore (I)' translated
		target: self
		selector: #exploreItem.
! !

!ABAbstractNode methodsFor: 'accessing' stamp: 'ThierryGoubier 7/19/2012 22:36'!
parentNode
	^ parentNode! !

!ABAbstractNode methodsFor: 'accessing' stamp: 'ThierryGoubier 7/19/2012 22:36'!
parentNode: anObject
	parentNode := anObject! !

!ABAbstractNode methodsFor: 'commands' stamp: 'ThierryGoubier 7/25/2012 23:19'!
saveCommands
	<textAreaCommand>
	^ {ABAcceptTextCommand.
	ABCancelTextCommand}! !

!ABAbstractNode methodsFor: 'querying' stamp: 'ThierryGoubier 7/19/2012 23:20'!
sourceClass
	"Should return a class if appropriate. The class is to provide the context for compilation, evaluation or styling."

	^ nil! !

!ABAbstractNode methodsFor: 'querying' stamp: 'ThierryGoubier 7/19/2012 23:19'!
sourceCode
	"Should return a source code or a description if appropriate."
	
	^ ''! !

!ABAbstractNode methodsFor: 'commands' stamp: 'ThierryGoubier 7/25/2012 23:19'!
undoCommands
	<textAreaCommand>
	^ {ABUndoTextCommand.
	ABRedoTextCommand}! !

ABAbstractNode subclass: #ABClassCategoryNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser'!

!ABClassCategoryNode commentStamp: '<historical>' prior: 0!
An ABClassCategoryNode is for class categories.
!

!ABClassCategoryNode methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 7/16/2012 17:07'!
childNodeClassFromItem: anItem
	(anItem isKindOf: Class)
		ifTrue: [ ^ ABClassNode ]
		ifFalse: [ ^ self class ]! !

!ABClassCategoryNode methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 7/17/2012 13:08'!
childrenItems
	"Extract from the organisation the various classes. Ok. Our upper level should try to built a hierarchy with the category name, if applicable (via various upTo: $-). This code is ready for that (introduce an additional hierarchy)."

	| c o |
	c := Smalltalk organization categoriesMatching: self item.
	c size > 2
		ifTrue: [ ^ c ]
		ifFalse: [ 
			o := OrderedCollection new.
			c do: [ :e | (Smalltalk organization classesInCategory: e) do: [ :class | o add: class ] ].
			^ o ]! !

!ABClassCategoryNode methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 7/27/2012 10:25'!
findClassCategory: aCat
	"Return the node if the item matches. By default, search among the kids. Is lazy and will avoid triggering child building."

	item == aCat
		ifTrue: [ ^ self ]
		ifFalse: [ ^ nil ]! !

ABAbstractNode subclass: #ABClassCommentNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser'!

!ABClassCommentNode commentStamp: '<historical>' prior: 0!
An ABClassCommentNode is for a class comment, if any. No children.
!

!ABClassCommentNode methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 7/26/2012 14:23'!
accept: aText
	"This should change the class comment."

	self item comment: aText! !

!ABClassCommentNode methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 7/20/2012 07:10'!
asString
	^ self item hasComment
		ifTrue: [ 'comment' asText addAttribute: TextEmphasis italic ]
		ifFalse: [ 'comment' ]! !

!ABClassCommentNode methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 7/27/2012 14:30'!
findClassComment: aClass
	^ item == aClass
		ifTrue: [ self ]
		ifFalse: [ nil ]! !

!ABClassCommentNode methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 7/16/2012 20:36'!
sourceCode
	^ self item comment! !

ABAbstractNode subclass: #ABClassNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser'!

!ABClassNode commentStamp: '<historical>' prior: 0!
An ABClassNode represent a class!

ABClassNode subclass: #ABClassExtensionNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser'!

!ABClassExtensionNode commentStamp: '<historical>' prior: 0!
An ABClassExtensionNode is used when browsing the extension of a package. Restrict itself to the methods of the extension of the parent package.!

!ABClassExtensionNode methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 7/17/2012 13:37'!
asString
	^ self item asString! !

!ABClassExtensionNode methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 7/17/2012 13:39'!
childNodeClassFromItem: anItem
	^ ABMethodNode! !

!ABClassExtensionNode methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 7/17/2012 13:52'!
childrenItems
	^ (self parentNode item packageInfo extensionMethodsForClass: self item) collect: [ :e | e selector ]! !

!ABClassNode methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 7/26/2012 14:31'!
accept: aText
	^ [ 
	UndefinedObject evaluatorClass new
		evaluate: (ReadStream on: aText)
		in: nil
		to: nil
		notifying: nil
		ifFail: [ ^ nil ]
		logged: true ]
		on: OutOfScopeNotification
		do: [ :ex | ex resume: true ]! !

!ABClassNode methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 7/17/2012 10:33'!
asString
	^ self item isMeta
		ifTrue: [ '- Class' ]
		ifFalse: [ self item asString ]! !

!ABClassNode methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 7/16/2012 20:07'!
childNodeClassFromItem: anItem
	anItem == self item classSide
		ifTrue: [ ^ self class ].
	anItem == self item
		ifTrue: [ ^ ABClassCommentNode ].
	^ ABMethodCategoryNode! !

!ABClassNode methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 7/16/2012 20:06'!
childrenItems
	^ self item isMeta
		ifFalse: [ 
			{(self item classSide).
			(self item)} , self item organization categories ]
		ifTrue: [ self item organization categories ]! !

!ABClassNode methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 7/16/2012 13:56'!
classSide
	^ self item isMeta
		ifFalse: [ {(self item classSide)} ]
		ifTrue: [ {} ]! !

!ABClassNode methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 7/27/2012 11:35'!
findClass: aClass
	^ self item == aClass
		ifTrue: [ self ]
		ifFalse: [ nil ]! !

!ABClassNode methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 7/19/2012 23:23'!
isStyled
	^ true! !

!ABClassNode methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 7/17/2012 13:54'!
privateClass
	^ self item! !

!ABClassNode methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 7/16/2012 14:18'!
sourceCode
	^ self item definition! !

ABAbstractNode subclass: #ABMethodCategoryNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser'!

!ABMethodCategoryNode commentStamp: '<historical>' prior: 0!
For a method category. No need for an -- all ---, the tree list allow for opening everything if needed. The 'as yet unclassified' method category (protocol) is managed on the fly (and exist in the system) !

!ABMethodCategoryNode methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 7/16/2012 00:18'!
childNodeClassFromItem: anItem
	^ ABMethodNode! !

!ABMethodCategoryNode methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 7/27/2012 16:25'!
childrenItems
	^ (self parentNode item methodsInCategory: self item) asSortedCollection! !

!ABMethodCategoryNode methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 7/27/2012 14:27'!
findMethodCategory: aMethodCategory
	^ aMethodCategory = item
		ifTrue: [ self ]
		ifFalse: [ nil ]! !

!ABMethodCategoryNode methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 7/17/2012 13:54'!
privateClass
	^ self parentNode privateClass! !

!ABMethodCategoryNode methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 7/17/2012 13:53'!
sourceClass
	^ self parentNode item! !

ABAbstractNode subclass: #ABMethodNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser'!

!ABMethodNode commentStamp: '<historical>' prior: 0!
Represent a method. Should be used to get the method source, selector and whatever.!

!ABMethodNode methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 7/27/2012 13:42'!
findMethod: anItem
	^ item == anItem
		ifTrue: [ self ]
		ifFalse: [ nil ]! !

!ABMethodNode methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 7/19/2012 23:22'!
isStyled
	^ true! !

!ABMethodNode methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 7/17/2012 13:55'!
sourceClass
	"Get the class through the parent."

	^ (self parentNode privateClass compiledMethodAt: self item) methodClass! !

!ABMethodNode methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 7/17/2012 13:55'!
sourceCode
	"Get the source code through the parent."

	^ (self parentNode privateClass compiledMethodAt: self item) sourceCode! !

ABAbstractNode subclass: #ABPackageCategoryNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser'!

!ABPackageCategoryNode commentStamp: '<historical>' prior: 0!
An ABPackageCategoryNode represent a category of packages.

item contains an association "package category name" -> "list of packages.".

!

!ABPackageCategoryNode methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 7/14/2012 23:17'!
asString
	^ item key! !

!ABPackageCategoryNode methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 7/15/2012 13:58'!
childNodeClassFromItem: anItem
	^ ABPackageNode! !

!ABPackageCategoryNode methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 7/14/2012 23:18'!
childrenItems
	^ item value! !

ABAbstractNode subclass: #ABPackageNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser'!

!ABPackageNode commentStamp: '<historical>' prior: 0!
An ABPackageNode represent a package and all classes matching it (categories).
!

ABPackageNode subclass: #ABPackageExtensionNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser'!

!ABPackageExtensionNode commentStamp: '<historical>' prior: 0!
An ABPackageExtensionNode is for the extensions part of a package.!

!ABPackageExtensionNode methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 7/17/2012 13:31'!
asString
	^ '*extensions*'! !

!ABPackageExtensionNode methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 7/17/2012 13:38'!
childNodeClassFromItem: anItem
	^ ABClassExtensionNode! !

!ABPackageExtensionNode methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 7/17/2012 13:28'!
childrenItems
	^ self item packageInfo extensionClasses! !

!ABPackageNode methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 7/15/2012 07:49'!
asString
	^ self item packageName! !

!ABPackageNode methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 7/17/2012 13:30'!
childNodeClassFromItem: anItem
	((anItem isKindOf: Class) or: [ anItem isKindOf: Metaclass ])
		ifTrue: [ ^ ABClassNode ].
	(anItem isKindOf: MCWorkingCopy)
		ifTrue: [ ^ ABPackageExtensionNode ].
	^ ABClassCategoryNode! !

!ABPackageNode methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 7/17/2012 13:23'!
childrenItems
	"Extract from the organisation the various classes."

	| c o |
	c := self item packageInfo systemCategories.
	o := OrderedCollection new.
	o add: self item.
	c size = 1
		ifTrue: [ c do: [ :e | (Smalltalk organization classesInCategory: e) do: [ :class | o add: class ] ] ]
		ifFalse: [ o addAll: (c" reject: [ :e | Smalltalk organization isEmptyCategoryNamed: e ]") ].
	^ o! !

!ABPackageNode methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 7/19/2012 23:29'!
sourceCode
	^ self item description! !

TreeModel subclass: #ABTreeModel
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser'!

!ABTreeModel commentStamp: '<historical>' prior: 0!
An ABTreeModel is an extension of TreeModel to handle drag and drop.

My problem with the drag and drop is that I loose the wrapper object (and therefore the logic) which is needed when the drag starts. I can :
* add another level of wrapper object below the list objects.
* have only one type of list wrapper object and my own wrappers.
* subclass the PluggableTreeMorph.!

!ABTreeModel class methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 7/16/2012 23:27'!
defaultSpec
	<spec>
	
	^ {#TreeSpec.
	    		#model:.							#model.
			#getLabelSelector:.				#wrapItem:.
			#getRootsSelector:.				#getRoots.
			#getChildrenSelector:.				#getChildrenFor:.
			#setSelectedSelector:.			#setSelectedItem:.
			#color:.							Color white.
			#getMenuSelector:.				#getMenu:.
			#hResizing:. 						#spaceFill.
			#vResizing:. 						#spaceFill.
			#eventHandler:.					{	#EventHandler. #on:send:to:. #keyStroke.	#keyStroke:fromMorph:. #model	}.
			#dropItemSelector:.				#dropItem:on:.
			#dragItemSelector:.				#dragItem:.}! !

!ABTreeModel methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 7/16/2012 23:29'!
dragItem: anItem
	(anItem isKindOf: Association)
		ifTrue: [ ^ anItem key ].
	^ anItem! !

!ABTreeModel methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 7/16/2012 23:27'!
dropItem: anItem on: aTarget
	^nil! !

!ABTreeModel methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 7/27/2012 17:20'!
rootsAdd: aNode
	"Algorithm: look for the parent of the node. If this parent node is found, then add a node equivalent to the contents if they do exist."

	| pNode |
	(pNode := aNode parentNode) isNil
		ifTrue: [ rootsHolder add: aNode ]
		ifFalse: [ 
			rootsHolder contents
				do: [ :e | 
					| r |
					(r := e findItem: pNode) notNil
						ifTrue: [ 
							| n wNode |
							r hasContents
								ifTrue: [ 
									wNode := r childNodeFromItem: aNode.
									r contents add: wNode ].
							(n := (self widget scroller submorphs at: 1) findItem: r) notNil
								ifTrue: [ 
									n hasContents
										ifTrue: [ n addItem: (ABTreeNodeModel with: wNode model: wNode) ] ].
								^ self ] ] ]! !

!ABTreeModel methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 7/27/2012 17:24'!
rootsRemove: aNode
	"Beware : a node is not an instance of ABTreeNodeModel"

	rootsHolder contents
		do: [ :e | 
			| r |
			(r := e findItem: aNode) notNil
				ifTrue: [ 
					r == e
						ifTrue: [ rootsHolder remove: r ]
						ifFalse: [ r parentNode contents remove: r ].
					self assert: ((self widget scroller submorphs at: 1) removeItem: r) notNil ] ]! !

TextModel subclass: #ABTextModel
	instanceVariableNames: 'menuBlockHolder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser'!

!ABTextModel commentStamp: '<historical>' prior: 0!
An ABTextModel is an extension to TextModel allowing the override of the menu.
!

!ABTextModel class methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 7/17/2012 22:42'!
defaultSpec
	<spec>
	
	^ SpecLayout text
			send: #on:text:accept:readSelection:menu: withArguments: #(model getText accept:notifying: readSelection getMenu:);
			send: #classOrMetaClass: withArguments: {#(model behavior)};
			send: #enabled: withArguments: {#(model enabled)};
			send: #eventHandler: withArguments: {#(EventHandler on:send:to: keyStroke keyStroke:fromMorph: model)};
			send: #vSpaceFill;
			send: #hSpaceFill;
			yourself! !

!ABTextModel methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 7/17/2012 22:39'!
getMenu: aMenu

	menuBlockHolder contents cull: aMenu cull: self.
	^ aMenu! !

!ABTextModel methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 7/17/2012 22:40'!
initialize
	"Initialization code for TreeModel"

	super initialize.

	menuBlockHolder := [] asValueHolder.! !

!ABTextModel methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 7/17/2012 22:39'!
menu: aBlock
	"Set the block used to generate the tree menu"

	menuBlockHolder contents: aBlock! !

MorphTreeModel subclass: #ABPackageList
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser'!

!ABPackageList commentStamp: '<historical>' prior: 0!
An ABPackageList is in charge of displaying a tree of Packages / Category / Classes / Protocols / Methods.

It's a non-Spec based attempt.

ABPackageList open!

!ABPackageList class methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 7/16/2012 00:42'!
defaultPackageCategoriesNames
	"As it says: categories for packages who don't use the - convention."
	
	^ #('Configuration' 'Nautilus' 'Text' 'Shout' 'Collections' 'Fuel' 'Morphic' 'Monticello' 'Tools' 'Compiler' 'Network' 'Compression' 'Balloon' 'Kernel' 'FreeType' 'SmaCC')! !

!ABPackageList class methodsFor: 'instance creation' stamp: 'ThierryGoubier 7/14/2012 11:10'!
open
	^ self new open! !

!ABPackageList class methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 7/19/2012 22:40'!
rootItems
	"This is where I should rework the organisation. Use broad classes to reduce the number of top-level items. To be improved. Does not catch classes which have no "

	| l o |
	l := MCWorkingCopy allManagers asOrderedCollection.
	o := OrderedCollection new.
	self defaultPackageCategoriesNames
		do: [ :w | 
			| i |
			i := w -> OrderedCollection new.
			(l select: [ :e | e packageName size >= w size and: [ (e packageName copyFrom: 1 to: w size) = w ] ])
				do: [ :e | 
					l remove: e.
					i value add: e ].
			o add: i ].
	[ l notEmpty ]
		whileTrue: [ 
			| p w i |
			p := l first.
			w := p packageName copyUpTo: $-.
			i := w -> (OrderedCollection with: p).
			o add: i.
			l remove: p.
			(l select: [ :e | (e packageName copyUpTo: $-) = w ])
				do: [ :e | 
					l remove: e.
					i value add: e ] ].
	^ o collect: [ :e | ABPackageCategoryNode with: e ]! !

!ABPackageList methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 7/14/2012 11:11'!
defaultTreeMorph
	^ super defaultTreeMorph
		doubleClickSelector: #doubleClick;
		nodeStringGetter: #asString;
		getMenuSelector: #menu:shifted:! !

!ABPackageList methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 7/14/2012 11:11'!
doubleClick
	self selectedNode ifNotNil: [:n | n doubleClick]! !

!ABPackageList methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 7/14/2012 11:11'!
keyStroke: anEvent from: aTreeMorph
	self selectedNode ifNotNil: [ :current | current keyStroke: anEvent from: aTreeMorph ]! !

!ABPackageList methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 7/14/2012 11:11'!
menu: menu shifted: b
	"Set up the menu to apply to the receiver's, 
	honoring the #shifted boolean"

	self selectedNode ifNotNil: [ :current | current menu: menu shifted: b ].
	^ menu! !

!ABPackageList methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 7/14/2012 11:12'!
open
	^ self openDialogWindowIn: World title: 'KSBrowser'.! !

!ABPackageList methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 7/15/2012 21:34'!
rootItems
	^ self class rootItems! !

!ABPackageList methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 7/15/2012 13:58'!
rootNodeClassFromItem: anItem
	^ ABPackageCategoryNode! !

MorphTreeNodeModel subclass: #ABTreeNodeModel
	instanceVariableNames: 'contents'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser'!

!ABTreeNodeModel commentStamp: '<historical>' prior: 0!
An ABTreeNodeModel represent a node for the tree. It reroutes most of it's request to the underlying item.

It caches its contents however.!

!ABTreeNodeModel methodsFor: 'querying' stamp: 'ThierryGoubier 7/25/2012 23:23'!
buildTextMenu: aMenu on: aTarget with: aRequestor
	^ self item buildTextMenu: aMenu on: aTarget with: aRequestor! !

!ABTreeNodeModel methodsFor: 'accessing' stamp: 'ThierryGoubier 7/19/2012 22:33'!
childrenItems
	^ item contents! !

!ABTreeNodeModel methodsFor: 'accessing' stamp: 'ThierryGoubier 7/19/2012 22:26'!
contents
	contents isNil
		ifTrue: [ contents := super contents ].
	^ contents! !

!ABTreeNodeModel methodsFor: 'querying' stamp: 'ThierryGoubier 7/26/2012 23:16'!
findItem: anItem
	"searching for an item, returns a node."

	self item == anItem
		ifTrue: [ ^ self ].
	contents notNil
		ifTrue: [ 
			contents
				do: [ :e | 
					| s |
					(s := e findItem: anItem) notNil
						ifTrue: [ ^ s ] ] ].
	^ nil! !

!ABTreeNodeModel methodsFor: 'testing' stamp: 'ThierryGoubier 7/26/2012 21:41'!
hasContents
	^ contents notNil! !

!ABTreeNodeModel methodsFor: 'accessing' stamp: 'ThierryGoubier 7/19/2012 23:24'!
isStyled
	^ item isStyled! !

!ABTreeNodeModel methodsFor: 'accessing' stamp: 'ThierryGoubier 7/19/2012 22:27'!
sourceClass
	^ item sourceClass! !

!ABTreeNodeModel methodsFor: 'accessing' stamp: 'ThierryGoubier 7/19/2012 22:27'!
sourceCode
	^ item sourceCode! !

!IndentingListItemMorph methodsFor: '*Alt-Browser' stamp: 'ThierryGoubier 7/27/2012 15:39'!
addItem: aNode
	"Add a submorph for that item."

	| newMorph lc |
	lc := self lastChild.
	newMorph := container indentingItemClass basicNew
		initWithContents: aNode
		prior: lc
		forList: container
		indentLevel: lc indentLevel.
	lc nextSibling: newMorph.
	self isExpanded
		ifTrue: [ 
			container scroller addAllMorphs: {newMorph} after: lc.
			container adjustSubmorphPositions ]! !

!IndentingListItemMorph methodsFor: '*Alt-Browser' stamp: 'ThierryGoubier 7/27/2012 14:35'!
findItem: aNode
	| found |
	self
		withSiblingsDo: [ :each | 
			each complexContents item == aNode
				ifTrue: [ ^ each ] ].
	self
		withSiblingsDo: [ :each | 
			each firstChild notNil
				ifTrue: [ 
					found := each firstChild findItem: aNode.
					found notNil
						ifTrue: [ ^ found ] ] ].
	^ nil! !

!IndentingListItemMorph methodsFor: '*Alt-Browser' stamp: 'ThierryGoubier 7/27/2012 14:38'!
hasContents
	^ firstChild notNil! !

!IndentingListItemMorph methodsFor: '*Alt-Browser' stamp: 'ThierryGoubier 7/27/2012 17:03'!
removeItem: aNode
	| found |
	found := self findItem: aNode.
	found isNil
		ifFalse: [ 
			found recursiveDelete.
			container adjustSubmorphPositions ].
	^ found! !

Object subclass: #ABCommand
	instanceVariableNames: 'target requestor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABCommand commentStamp: '<historical>' prior: 0!
An ABCommand is in charge of preparing and executing commands on the Alt Browser.

Instance Variables
	requestor:	<Object>	The object sending the command?
	target:		<Object>	The target (where the command is applied)!

ABCommand subclass: #ABAcceptTextCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABAcceptTextCommand commentStamp: '<historical>' prior: 0!
An ABAcceptTextCommand is about accepting the text.
!

!ABAcceptTextCommand class methodsFor: 'testing' stamp: 'lr 12/10/2010 09:25'!
takesNodes
	^ false! !

!ABAcceptTextCommand class methodsFor: 'testing' stamp: 'lr 12/10/2010 09:25'!
takesText
	^ true! !

!ABAcceptTextCommand methodsFor: 'execution' stamp: 'ThierryGoubier 7/26/2012 14:20'!
execute
	target widget accept.
	requestor selectedItem item accept: target getText! !

!ABAcceptTextCommand methodsFor: 'accessing' stamp: 'lr 12/10/2010 10:39'!
group
	^ #zip! !

!ABAcceptTextCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 7/17/2012 23:17'!
icon
	^ #smallOkIcon! !

!ABAcceptTextCommand methodsFor: 'testing' stamp: 'lr 12/10/2010 09:21'!
isActive
	^ true! !

!ABAcceptTextCommand methodsFor: 'accessing' stamp: 'lr 12/10/2010 10:30'!
keystroke
	^ $s! !

!ABAcceptTextCommand methodsFor: 'accessing' stamp: 'lr 12/10/2010 09:09'!
label
	^ 'Accept'! !

ABCommand subclass: #ABCancelTextCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABCancelTextCommand commentStamp: '<historical>' prior: 0!
An ABCancelTextCommand is about cancelling the last action.!

!ABCancelTextCommand class methodsFor: 'testing' stamp: 'lr 12/10/2010 09:25'!
takesNodes
	^ false! !

!ABCancelTextCommand class methodsFor: 'testing' stamp: 'lr 12/10/2010 09:25'!
takesText
	^ true! !

!ABCancelTextCommand methodsFor: 'execution' stamp: 'lr 12/10/2010 09:11'!
execute
	target cancel! !

!ABCancelTextCommand methodsFor: 'accessing' stamp: 'lr 12/10/2010 10:39'!
group
	^ #zip! !

!ABCancelTextCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 7/17/2012 23:17'!
icon
	^ #smallCancelIcon! !

!ABCancelTextCommand methodsFor: 'testing' stamp: 'lr 12/10/2010 09:22'!
isActive
	^ true! !

!ABCancelTextCommand methodsFor: 'accessing' stamp: 'lr 12/10/2010 09:11'!
keystroke
	^ $l! !

!ABCancelTextCommand methodsFor: 'accessing' stamp: 'lr 12/10/2010 09:11'!
label
	^ 'Cancel'! !

!ABCommand class methodsFor: 'instance creation' stamp: 'cwp 10/14/2006 23:29'!
on: target for: requestor
	^ self new setTarget: target requestor: requestor! !

!ABCommand class methodsFor: 'testing' stamp: 'cwp 10/15/2006 00:58'!
takesNodes
	^ true! !

!ABCommand class methodsFor: 'testing' stamp: 'cwp 10/15/2006 00:58'!
takesText
	^ false! !

!ABCommand methodsFor: 'user interface' stamp: 'ThierryGoubier 7/25/2012 23:27'!
addItemToMenu: aMenu
	self isActive
		ifTrue: [ 
			(aMenu add: self label target: self selector: #execute)
				enablementSelector: #isEnabled;
				icon:
						(self icon notNil
								ifTrue: [ UITheme current perform: self icon ]
								ifFalse: [ nil ]);
				keyText: self keyString ]! !

!ABCommand methodsFor: 'accessing' stamp: 'lr 4/25/2007 09:27'!
buttonLabel
	^ self label! !

!ABCommand methodsFor: 'accessing' stamp: 'cwp 9/30/2007 18:59'!
cluster
	^ nil! !

!ABCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 7/17/2012 20:52'!
createCluster
	^ ABCommandCluster label: self cluster! !

!ABCommand methodsFor: 'execution' stamp: 'cwp 10/15/2006 14:12'!
execute
	! !

!ABCommand methodsFor: 'accessing' stamp: 'cwp 10/6/2006 21:44'!
group
	^ #general! !

!ABCommand methodsFor: 'accessing' stamp: 'cwp 10/6/2006 18:45'!
icon
	^ nil! !

!ABCommand methodsFor: 'testing' stamp: 'cwp 10/15/2006 22:05'!
isActive
	^ false! !

!ABCommand methodsFor: 'testing' stamp: 'cwp 10/15/2006 19:31'!
isEnabled
	^ true! !

!ABCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 7/17/2012 23:23'!
keyString
	| c |
	^ (c := self keystroke) isNil
		ifFalse: [ 'Ctrl-' , c asString ]
		ifTrue: [nil]! !

!ABCommand methodsFor: 'accessing' stamp: 'cwp 9/27/2006 22:53'!
keystroke
	^ nil
! !

!ABCommand methodsFor: 'accessing' stamp: 'cwp 9/27/2006 22:53'!
label
	self subclassResponsibility
! !

!ABCommand methodsFor: 'accessing' stamp: 'dc 8/3/2007 12:50'!
longDescription
	"Override this and return a string or a text for the fly-by-help (tooltip)"
	^ nil! !

!ABCommand methodsFor: 'accessing' stamp: 'lr 9/24/2010 21:46'!
longLabel
	^ self cluster isNil
		ifTrue: [ self label ]
		ifFalse: [ self cluster , ' > ' , self label ]! !

!ABCommand methodsFor: 'accessing' stamp: 'cwp 10/14/2006 23:39'!
on: anObject for: aRequestor
	^ self class on: anObject for: aRequestor! !

!ABCommand methodsFor: 'user interface' stamp: 'dc 1/3/2008 10:11'!
order
	^ self label! !

!ABCommand methodsFor: 'user interface' stamp: 'cwp 10/15/2006 12:37'!
perform: aSelector orSendTo: anObject
	self perform: aSelector! !

!ABCommand methodsFor: 'user interface' stamp: 'lr 11/25/2010 23:45'!
select: aNode with: anAnnouncer
	aNode announceSelectionWith: anAnnouncer! !

!ABCommand methodsFor: 'initialize-release' stamp: 'lr 7/3/2009 22:27'!
setTarget: anObject requestor: aRequestor 
	target := anObject.
	requestor := aRequestor! !

!ABCommand methodsFor: 'testing' stamp: 'lr 2/7/2010 12:50'!
takesNodes
	^ self class takesNodes! !

!ABCommand methodsFor: 'testing' stamp: 'lr 2/7/2010 12:50'!
takesText
	^ self class takesText! !

!ABCommand methodsFor: 'testing' stamp: 'cwp 9/27/2006 22:53'!
wantsButton
	"Put this here for compatibility. Eventually this will be obsolete"

	^ false
! !

!ABCommand methodsFor: 'testing' stamp: 'lr 8/8/2010 10:47'!
wantsMenu
	"Put this here for compatibility. Eventually this will be obsolete"

	^ true
! !

ABCommand subclass: #ABCopyTextCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABCopyTextCommand class methodsFor: 'testing' stamp: 'lr 12/10/2010 09:25'!
takesNodes
	^ false! !

!ABCopyTextCommand class methodsFor: 'testing' stamp: 'lr 12/10/2010 09:25'!
takesText
	^ true! !

!ABCopyTextCommand methodsFor: 'execution' stamp: 'lr 12/10/2010 10:40'!
execute
	target copySelection! !

!ABCopyTextCommand methodsFor: 'accessing' stamp: 'lr 12/10/2010 10:37'!
group
	^ #clipboard! !

!ABCopyTextCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 7/17/2012 23:17'!
icon
	^ #smallCopyIcon! !

!ABCopyTextCommand methodsFor: 'testing' stamp: 'lr 12/10/2010 09:22'!
isActive
	^ true! !

!ABCopyTextCommand methodsFor: 'testing' stamp: 'ThierryGoubier 7/26/2012 07:46'!
isEnabled
	^ target widget trueSelection isEmpty not! !

!ABCopyTextCommand methodsFor: 'accessing' stamp: 'lr 12/10/2010 09:11'!
keystroke
	^ $c! !

!ABCopyTextCommand methodsFor: 'accessing' stamp: 'lr 12/10/2010 09:11'!
label
	^ 'Copy'! !

!ABCopyTextCommand methodsFor: 'accessing' stamp: 'lr 12/10/2010 09:29'!
order
	^ '2'! !

ABCommand subclass: #ABCutTextCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABCutTextCommand class methodsFor: 'testing' stamp: 'lr 12/10/2010 09:25'!
takesNodes
	^ false! !

!ABCutTextCommand class methodsFor: 'testing' stamp: 'lr 12/10/2010 09:25'!
takesText
	^ true! !

!ABCutTextCommand methodsFor: 'execution' stamp: 'lr 3/6/2011 15:03'!
execute
	target cut! !

!ABCutTextCommand methodsFor: 'accessing' stamp: 'lr 12/10/2010 09:27'!
group
	^ #clipboard! !

!ABCutTextCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 7/17/2012 23:17'!
icon
	^ #smallCutIcon! !

!ABCutTextCommand methodsFor: 'testing' stamp: 'lr 12/10/2010 09:22'!
isActive
	^ true! !

!ABCutTextCommand methodsFor: 'testing' stamp: 'ThierryGoubier 7/26/2012 07:49'!
isEnabled
	^ target widget trueSelection isEmpty not! !

!ABCutTextCommand methodsFor: 'accessing' stamp: 'lr 12/10/2010 09:12'!
keystroke
	^ $x! !

!ABCutTextCommand methodsFor: 'accessing' stamp: 'lr 12/10/2010 09:12'!
label
	^ 'Cut'! !

!ABCutTextCommand methodsFor: 'accessing' stamp: 'lr 12/10/2010 09:29'!
order
	^ '1'! !

ABCommand subclass: #ABDoItTextCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABDoItTextCommand commentStamp: '<historical>' prior: 0!
ABDoItTextCommand : execute the code selected
!

ABDoItTextCommand subclass: #ABDebugItTextCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABDebugItTextCommand class methodsFor: 'testing' stamp: 'lr 8/27/2011 10:02'!
takesNodes
	^ false! !

!ABDebugItTextCommand class methodsFor: 'testing' stamp: 'lr 8/27/2011 10:02'!
takesText
	^ true! !

!ABDebugItTextCommand methodsFor: 'execution' stamp: 'lr 8/27/2011 09:58'!
execute
	target debugIt! !

!ABDebugItTextCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 7/17/2012 23:17'!
icon
	^ #smallDebugIcon! !

!ABDebugItTextCommand methodsFor: 'testing' stamp: 'lr 8/27/2011 09:58'!
isActive
	^ true! !

!ABDebugItTextCommand methodsFor: 'accessing' stamp: 'lr 8/27/2011 09:58'!
keystroke
	^ $D! !

!ABDebugItTextCommand methodsFor: 'accessing' stamp: 'lr 8/27/2011 09:58'!
label
	^ 'Debug it'! !

!ABDebugItTextCommand methodsFor: 'accessing' stamp: 'lr 8/27/2011 10:00'!
order
	^ '5'! !

!ABDoItTextCommand class methodsFor: 'testing' stamp: 'lr 3/6/2011 14:55'!
takesNodes
	^ false! !

!ABDoItTextCommand class methodsFor: 'testing' stamp: 'lr 3/6/2011 14:55'!
takesText
	^ true! !

!ABDoItTextCommand methodsFor: 'execution' stamp: 'ThierryGoubier 7/26/2012 10:41'!
execute
	"Treat the current selection as an expression; evaluate it."

	| result rcvr ctxt |
	(requestor selectedItem item respondsTo: #doItReceiver)
		ifTrue: [ 
			rcvr := requestor selectedItem item doItReceiver.
			ctxt := requestor selectedItem item doItContext ]
		ifFalse: [ rcvr := ctxt := nil ].
	result := [ 
	rcvr class evaluatorClass new
		evaluate: (ReadStream on: target widget trueSelection)
		in: ctxt
		to: rcvr
		notifying: target widget
		ifFail: [ 
			target widget flash.
			^ nil ]
		logged: true	"don't set it to self.. The receiver can be destroyed as a result of evaluateSelection" ]
		on: OutOfScopeNotification
		do: [ :ex | ex resume: true ].
	^ result! !

!ABDoItTextCommand methodsFor: 'accessing' stamp: 'lr 3/6/2011 14:55'!
group
	^ #smalltalk! !

!ABDoItTextCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 7/17/2012 23:18'!
icon
	^ #smallDoItIcon! !

!ABDoItTextCommand methodsFor: 'testing' stamp: 'ThierryGoubier 7/25/2012 23:25'!
isActive
	^ target readSelection notNil! !

!ABDoItTextCommand methodsFor: 'testing' stamp: 'ThierryGoubier 7/26/2012 10:40'!
isEnabled
	^ target widget trueSelection notEmpty! !

!ABDoItTextCommand methodsFor: 'accessing' stamp: 'lr 3/17/2011 21:38'!
keystroke
	^ $d! !

!ABDoItTextCommand methodsFor: 'accessing' stamp: 'lr 3/6/2011 14:55'!
label
	^ 'Do it'! !

!ABDoItTextCommand methodsFor: 'accessing' stamp: 'lr 3/6/2011 14:54'!
order
	^ '1'! !

ABDoItTextCommand subclass: #ABExploreItTextCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABExploreItTextCommand class methodsFor: 'testing' stamp: 'lr 3/6/2011 15:08'!
takesNodes
	^ false! !

!ABExploreItTextCommand class methodsFor: 'testing' stamp: 'lr 3/6/2011 15:08'!
takesText
	^ true! !

!ABExploreItTextCommand methodsFor: 'execution' stamp: 'ThierryGoubier 7/26/2012 13:47'!
execute
	super execute explore! !

!ABExploreItTextCommand methodsFor: 'testing' stamp: 'lr 3/6/2011 15:05'!
isActive
	^ true! !

!ABExploreItTextCommand methodsFor: 'accessing' stamp: 'lr 3/6/2011 15:05'!
keystroke
	^ $I! !

!ABExploreItTextCommand methodsFor: 'accessing' stamp: 'lr 3/6/2011 15:05'!
label
	^ 'Explore it'! !

!ABExploreItTextCommand methodsFor: 'accessing' stamp: 'lr 3/6/2011 15:05'!
order
	^ '4'! !

ABDoItTextCommand subclass: #ABInspectItTextCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABInspectItTextCommand class methodsFor: 'testing' stamp: 'lr 3/6/2011 14:55'!
takesNodes
	^ false! !

!ABInspectItTextCommand class methodsFor: 'testing' stamp: 'lr 3/6/2011 14:55'!
takesText
	^ true! !

!ABInspectItTextCommand methodsFor: 'execution' stamp: 'ThierryGoubier 7/26/2012 13:48'!
execute
	super execute inspect! !

!ABInspectItTextCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 7/17/2012 23:18'!
icon
	^ #smallInspectItIcon! !

!ABInspectItTextCommand methodsFor: 'testing' stamp: 'lr 3/6/2011 14:55'!
isActive
	^ true! !

!ABInspectItTextCommand methodsFor: 'accessing' stamp: 'lr 3/17/2011 21:38'!
keystroke
	^ $i! !

!ABInspectItTextCommand methodsFor: 'accessing' stamp: 'lr 3/6/2011 14:56'!
label
	^ 'Inspect it'! !

!ABInspectItTextCommand methodsFor: 'accessing' stamp: 'lr 3/6/2011 14:56'!
order
	^ '3'! !

ABDoItTextCommand subclass: #ABPrintItTextCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABPrintItTextCommand class methodsFor: 'testing' stamp: 'lr 3/6/2011 14:55'!
takesNodes
	^ false! !

!ABPrintItTextCommand class methodsFor: 'testing' stamp: 'lr 3/6/2011 14:55'!
takesText
	^ true! !

!ABPrintItTextCommand methodsFor: 'execution' stamp: 'lr 3/6/2011 15:13'!
execute
	target printIt! !

!ABPrintItTextCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 7/17/2012 23:18'!
icon
	^ #smallPrintIcon! !

!ABPrintItTextCommand methodsFor: 'testing' stamp: 'lr 3/6/2011 14:55'!
isActive
	^ true! !

!ABPrintItTextCommand methodsFor: 'accessing' stamp: 'lr 3/6/2011 14:56'!
keystroke
	^ $p! !

!ABPrintItTextCommand methodsFor: 'accessing' stamp: 'lr 3/6/2011 14:56'!
label
	^ 'Print it'! !

!ABPrintItTextCommand methodsFor: 'accessing' stamp: 'lr 3/6/2011 14:56'!
order
	^ '2'! !

ABCommand subclass: #ABPasteTextCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABPasteTextCommand commentStamp: '<historical>' prior: 0!
An ABPasteTextCommand : Paste text from Clipboard. Directly target the editor; neither the model nor the widget have methods for handling that.
!

!ABPasteTextCommand class methodsFor: 'testing' stamp: 'lr 12/10/2010 09:25'!
takesNodes
	^ false! !

!ABPasteTextCommand class methodsFor: 'testing' stamp: 'lr 12/10/2010 09:25'!
takesText
	^ true! !

!ABPasteTextCommand methodsFor: 'execution' stamp: 'ThierryGoubier 7/26/2012 08:03'!
execute
	target widget textMorph editor replace: target widget trueSelectionInterval with: Clipboard clipboardText and: [  ]! !

!ABPasteTextCommand methodsFor: 'accessing' stamp: 'lr 12/10/2010 09:27'!
group
	^ #clipboard! !

!ABPasteTextCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 7/17/2012 23:18'!
icon
	^ #smallPasteIcon! !

!ABPasteTextCommand methodsFor: 'testing' stamp: 'lr 12/10/2010 09:22'!
isActive
	^ true! !

!ABPasteTextCommand methodsFor: 'testing' stamp: 'ThierryGoubier 7/26/2012 07:52'!
isEnabled
	^ Clipboard clipboardText notEmpty! !

!ABPasteTextCommand methodsFor: 'accessing' stamp: 'lr 12/10/2010 09:14'!
keystroke
	^ $v! !

!ABPasteTextCommand methodsFor: 'accessing' stamp: 'lr 12/10/2010 09:14'!
label
	^ 'Paste'! !

!ABPasteTextCommand methodsFor: 'accessing' stamp: 'lr 12/10/2010 09:29'!
order
	^ '3'! !

ABCommand subclass: #ABRedoTextCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABRedoTextCommand commentStamp: '<historical>' prior: 0!
An ABRedoTextCommand calls the text editor redo command.
!

!ABRedoTextCommand class methodsFor: 'testing' stamp: 'lr 12/10/2010 09:25'!
takesNodes
	^ false! !

!ABRedoTextCommand class methodsFor: 'testing' stamp: 'lr 12/10/2010 09:25'!
takesText
	^ true! !

!ABRedoTextCommand methodsFor: 'execution' stamp: 'ThierryGoubier 7/26/2012 08:07'!
execute
	target widget textMorph editor redo! !

!ABRedoTextCommand methodsFor: 'accessing' stamp: 'lr 12/10/2010 10:38'!
group
	^ #adapting! !

!ABRedoTextCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 7/17/2012 23:18'!
icon
	^ #smallRedoIcon! !

!ABRedoTextCommand methodsFor: 'testing' stamp: 'lr 12/10/2010 09:22'!
isActive
	^ true! !

!ABRedoTextCommand methodsFor: 'accessing' stamp: 'lr 12/10/2010 09:20'!
keystroke
	^ $Z! !

!ABRedoTextCommand methodsFor: 'accessing' stamp: 'lr 12/10/2010 09:20'!
label
	^ 'Redo'! !

ABCommand subclass: #ABUndoTextCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABUndoTextCommand commentStamp: '<historical>' prior: 0!
An ABUndoTextCommand calls the text editor undo command.!

!ABUndoTextCommand class methodsFor: 'testing' stamp: 'lr 12/10/2010 09:25'!
takesNodes
	^ false! !

!ABUndoTextCommand class methodsFor: 'testing' stamp: 'lr 12/10/2010 09:25'!
takesText
	^ true! !

!ABUndoTextCommand methodsFor: 'execution' stamp: 'ThierryGoubier 7/26/2012 08:07'!
execute
	target widget textMorph editor undo! !

!ABUndoTextCommand methodsFor: 'accessing' stamp: 'lr 12/10/2010 10:38'!
group
	^ #adapting! !

!ABUndoTextCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 7/17/2012 23:19'!
icon
	^ #smallUndoIcon! !

!ABUndoTextCommand methodsFor: 'testing' stamp: 'lr 12/10/2010 09:22'!
isActive
	^ true! !

!ABUndoTextCommand methodsFor: 'accessing' stamp: 'lr 12/10/2010 09:20'!
keystroke
	^ $z! !

!ABUndoTextCommand methodsFor: 'accessing' stamp: 'lr 12/10/2010 09:20'!
label
	^ 'Undo'! !

!PluggableTextMorph methodsFor: '*Alt-Browser' stamp: 'ThierryGoubier 7/26/2012 07:48'!
trueSelection
	"Return the selection text as remembered by the PluggableTextMoprh, even if not in focus."

	^ textMorph editor text copyFrom: selectionInterval first to: selectionInterval last! !

!PluggableTextMorph methodsFor: '*Alt-Browser' stamp: 'ThierryGoubier 7/25/2012 23:58'!
trueSelectionInterval
	^ selectionInterval! !
